@@ -15,7 +15,6 @@
 /// \brief C++ Tracing in FST Format
 ///
 //=============================================================================
-// SPDIFF_OFF
 
 // clang-format off
 
@@ -66,8 +65,8 @@ VerilatedFst::~VerilatedFst() {
     if (m_strbuf) VL_DO_CLEAR(delete[] m_strbuf, m_strbuf = nullptr);
 }
 
-void VerilatedFst::open(const char* filename) VL_MT_UNSAFE {
-    m_assertOne.check();
+void VerilatedFst::open(const char* filename) VL_MT_SAFE_EXCLUDES(m_mutex) {
+    const VerilatedLockGuard lock(m_mutex);
     m_fst = fstWriterCreate(filename, 1);
     fstWriterSetPackType(m_fst, FST_WR_PT_LZ4);
     fstWriterSetTimescaleFromString(m_fst, timeResStr().c_str());  // lintok-begin-on-ref
@@ -94,18 +93,19 @@ void VerilatedFst::open(const char* filename) VL_MT_UNSAFE {
     m_code2symbol.clear();
 
     // Allocate string buffer for arrays
-    if (!m_strbuf) { m_strbuf = new char[maxBits() + 32]; }
+    if (!m_strbuf) m_strbuf = new char[maxBits() + 32];
 }
 
-void VerilatedFst::close() {
-    m_assertOne.check();
-    VerilatedTrace<VerilatedFst>::close();
+void VerilatedFst::close() VL_MT_SAFE_EXCLUDES(m_mutex) {
+    const VerilatedLockGuard lock(m_mutex);
+    VerilatedTrace<VerilatedFst>::closeBase();
     fstWriterClose(m_fst);
     m_fst = nullptr;
 }
 
-void VerilatedFst::flush() {
-    VerilatedTrace<VerilatedFst>::flush();
+void VerilatedFst::flush() VL_MT_SAFE_EXCLUDES(m_mutex) {
+    const VerilatedLockGuard lock(m_mutex);
+    VerilatedTrace<VerilatedFst>::flushBase();
     fstWriterFlushContext(m_fst);
 }
 
